// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License.
// To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/deed.en_US./
// Uses:
// - https://www.thingiverse.com/thing:27097		// oshw logo
// - http://www.thingiverse.com/thing:212041		// shape library
//
// change history:
// - 28-04-2014: Changed bottom rotor so it can be glued to top rotor.
//		 Added pin to top rotor. Replacement for handle
//               Changed screw holes in mounting plate for M3 screws and removed hole for head of screw.
//		 Added metal rings on bottom and top of rollers
//		 Increased house height for 2 metal rings
//		 Removed handle
// - 29-4-2014:  Add clearance between top rotor and lid (made top rotor smaller)
//		 Add clearance between axis and hole for axis in bottom rotor (hole in rotor bigger)
//

use <library.scad>;
use <oshw.scad>;

$fn = 128;				// In module roller is a circle command that should use a high $fn

mode = 8;
//	1: assembled
//	2: exploded
//	3: printable (if your printer is big enough)
//	4: house
//	5: top part rotor
//	6: bottom part rotor
//	7: roller
//	8: lid
//	9: clamp top

pipe_rad = 0.5*5.5;

roller_middle_rad = 0.5*16.2;		// radius of middle of roller
roller_top_rad = 0.5*15;		// radius of top (and bottom) of roller. roller_top_rad < roller_middle_rad
roller_height = 8;
roller_hole_rad = 0.5*3.6;
roller_count = 4;

house_inner_rad = 0.5*50;
house_outer_rad = 0.5*58;

clamp_width = 10;			// length of tube under clamp

pipe_clearance = 1.5;			// space between tube and entry in house.
fit_clearance = 0.5;			// space between two parts that should fit tightly. For example bottom and top rotor

rotor_height = 3;
metal_ring_height_small = 0.6;
metal_ring_height_large = 1.8;

axis_rad = 0.5*11;

mounting_plate_width = 58;
mounting_plate_length = 58;
mounting_plate_thickness = 3;

print_clearance = 5;
delta=0.05;

module mounting_plate()
{
	difference()
	{
		translate([0,0,0.5*mounting_plate_thickness])
			cube([mounting_plate_width, mounting_plate_length, mounting_plate_thickness], center=true);
		translate([0,0,-delta])
		{
			translate([0.5*mounting_plate_width-7, 0.5*mounting_plate_length-7, 0])
				cylinder(h=20, r=1.7);
			translate([-0.5*mounting_plate_width+7, 0.5*mounting_plate_length-7, 0])
				cylinder(h=20, r=1.7);
			translate([0.5*mounting_plate_width-7, -0.5*mounting_plate_length+7, 0])
				cylinder(h=20, r=1.7);
			translate([-0.5*mounting_plate_width+7, -0.5*mounting_plate_length+7, 0])
				cylinder(h=20, r=1.7);
		}
	}
}

module hose_entries()
{
	r=pipe_rad+2;
	s=house_inner_rad-r-10;

	translate([0,s,r+mounting_plate_thickness+rotor_height])
	{
		rotate([0,90,0])
			cylinder(h=house_outer_rad, r=r);
	}
	translate([0,-s,r+mounting_plate_thickness+rotor_height])
	{
		rotate([0,90,0])
			cylinder(h=house_outer_rad, r=r);
	}
}

module clamp_block(l, h, num_rings)
{
	step = clamp_width/(1+num_rings);

	difference()
	{
		translate([0,-1*l,0])
			cube([clamp_width,2*l,h]);
		translate([0-1,0.5*l,h])
			rotate([0,90,0])
				cylinder(h=clamp_width+2,r=pipe_rad);
		translate([0-1,-0.5*l,h])
			rotate([0,90,0])
				cylinder(h=clamp_width+2,r=pipe_rad);
	}
	for(i=[1:num_rings])
	{
		translate([i*step,0,0])
		{
			translate([0-1,0.5*l,h])
				rotate([0,0,90])
					rotate([90,0,0])
						bended_tube(rad_outer = 1, rad_inner = 0, angle=180, off_center = pipe_rad, printable=false);
			translate([0-1,-0.5*l,h])
				rotate([0,0,90])
					rotate([90,0,0])
						bended_tube(rad_outer = 1, rad_inner = 0, angle=180, off_center = pipe_rad, printable=false);
		}
	}
}

module clamp()
{
	l = 2*(house_inner_rad-pipe_rad-2-10);
	h = pipe_rad+2+mounting_plate_thickness+rotor_height;

	translate([0.5*mounting_plate_width, 0,0,])
	difference()
	{
		clamp_block(l, h, 2);
		translate([0.5*(clamp_width),0,-delta])
			screw_hole(head_rad = 3.5, length = 2*h, thick_rad = roller_hole_rad, wall_thickness = 2*h, thin_rad = 1);
	}
}

module clamp_top()
{
	l = 2*(house_inner_rad-pipe_rad-2-10);
	h = pipe_rad+2+mounting_plate_thickness+rotor_height;


	color([0,1,1])
	{
		translate([0.5*mounting_plate_width+clamp_width,0,2*h-mounting_plate_thickness])
			rotate([0,180,0])
			{
				difference()
				{
					clamp_block(l, h-mounting_plate_thickness, 2);
					translate([0.5*(clamp_width),0,-delta])
						cylinder(h=2*h, r=roller_hole_rad);
				}
			}
	}
}

module house()
{
	hh = 2*rotor_height+roller_height+2*metal_ring_height_small+mounting_plate_thickness+metal_ring_height_large;

	color([1,0,0])
	{
		difference()
		{
			union()
			{
				mounting_plate();
				cylinder(h=hh, r=house_outer_rad);	
			}
			translate([0,0,mounting_plate_thickness]) cylinder(h=hh+2, r=house_inner_rad);	
			translate([0,0,-1]) cylinder(h=hh+2, r=axis_rad);
			translate([-house_outer_rad-1,0,0.5*roller_height+mounting_plate_thickness+rotor_height])
				rotate([0,0,90])
					rotate([90,0,0])
						linear_extrude(height = 2, center = false, convexity = 10 )
							oshw_logo_2d(roller_height+2*rotor_height);
			translate([0,0,metal_ring_height_large])
				hose_entries();
		}
		translate([-delta,0,0])
			clamp();
	}
}

module roller()
{
	h_r = roller_height;
	r_t = roller_top_rad;
	r_m = roller_middle_rad;

	rl = (0.25*h_r*h_r+(r_m-r_t)*(r_m-r_t))/(2*(r_m-r_t));

	color([0,1,0])
	{
		translate([0,0,0.5*h_r])
			difference()
			{
				rotate_extrude(convexity=10)
					difference()
					{
						translate([r_m-rl,0,0])
							circle(r=rl, $fn=512);			// $fn should be high
						translate([-2*rl,-rl])
							square([2*rl,2*rl]);
						translate([-1,0.5*h_r])
							square([2*rl,2*rl]);
						translate([-1,-0.5*h_r-2*rl])
							square([2*rl,2*rl]);
					}
				cylinder(h=roller_height+2, r=roller_hole_rad, center=true);
			}
	}
}

module rollers()
{
	a = 360/roller_count;
	for(i=[0:1:roller_count-1])
	{
		rotate([0,0,a*i])
			translate([house_inner_rad-roller_middle_rad-pipe_clearance ,0,0])
				roller();
	}
}

module rotor_bottom()
{
	l = house_inner_rad-roller_middle_rad-pipe_clearance;
	a = 360/roller_count;
	h = roller_height+2*metal_ring_height_small+rotor_height;

	color([0,0,1])
	{
		difference()
		{
			cylinder(h=h, r=l+roller_top_rad);
			for(i=[0:roller_count-1])
			{
				rotate([0,0,a*i])
				{
					translate([l,0,-delta])
					{
						screw_hole(head_rad = 3.5, length = 20, thick_rad = roller_hole_rad, wall_thickness = rotor_height, thin_rad = 1);
						rotate([0,0,0.5*a])
							translate([0,0,rotor_height])
							{
								cylinder(h=h+2, r=roller_middle_rad+1);
							}
					}
				}
			}
			translate([0,0,rotor_height])
				ring(rad_outer=l+roller_middle_rad, rad_inner=l, height = h+2, printable=false);
			translate([0,0,-1])
				cylinder(h=h+2 , r=axis_rad+fit_clearance, $fn = 6 );
		}
	}
	
}

module rotor_top()
{
	l = house_inner_rad-roller_middle_rad-pipe_clearance;
	a = 360/roller_count;
	h = roller_height+2*metal_ring_height_small+2*rotor_height;

	color([0,0,1])
	{
		difference()
		{
			cylinder(h=rotor_height, r=l+roller_top_rad-2*fit_clearance);
			for(i=[0:roller_count-1])
			{
				rotate([0,0,a*i])
				{
					translate([l,0,-1])
						cylinder( h=rotor_height+2, r=roller_hole_rad );
				}
			}
		}
		//translate([0,0,-1])
			cylinder(h=h+20 , r=axis_rad, $fn = 6 );
	}
}

module lid()
{
	r_r = house_inner_rad-pipe_clearance;			// rotor radius (center to outside of rollers)

	color([1,1,0])
	{
		translate([0,0,-rotor_height])
		{
			difference()
			{
				union()
				{
					translate([0,0,rotor_height])
						mounting_plate();
					cylinder( h=rotor_height, r=house_inner_rad-fit_clearance );
				}
				translate([0,0,-mounting_plate_thickness-2])
				{
					cylinder(h=mounting_plate_thickness+rotor_height+2, r= r_r-fit_clearance);
				}
				translate([0,0,-1])
				{
					cylinder(h=mounting_plate_thickness+rotor_height+2, r= r_r-0.5*roller_top_rad+roller_hole_rad);
				}
			}
		}
	}
}

module assembled()
{
	house();
	clamp_top();
	hh = 2*rotor_height+roller_height+mounting_plate_thickness;
	translate([0,0,hh])
		lid();
	translate([0,0,mounting_plate_thickness])
		rotor_bottom();
	translate([0,0,2*rotor_height+mounting_plate_thickness+roller_height+2*metal_ring_height_small])
		rotate([180,0,0])
			rotor_top();
	translate([0,0,mounting_plate_thickness+rotor_height])
		rollers();
}

module exploded()
{
	hh = 2*rotor_height+roller_height+mounting_plate_thickness;
	house();
	translate([0,0,10])
		clamp_top();
	translate([0,0,30])
	{
		translate([0,0,mounting_plate_thickness])
			rotor_bottom();
		translate([0,0,10])
		{
			translate([0,0,mounting_plate_thickness+rotor_height])
				rollers();
			translate([0,0,30])
			{
				translate([0,0,roller_height+mounting_plate_thickness+rotor_height])
					rotate([180,0,0])
						rotor_top();
				translate([0,0,-20])
				{
					translate([0,0,hh])
						lid();
				}
			}
		}
	}
}


module printable()
{
	h = pipe_rad+2+mounting_plate_thickness+rotor_height;

	translate([0,0,0]) house();
	translate([print_clearance+mounting_plate_width+clamp_width,0,mounting_plate_thickness]) mirror([0,0,1]) lid();
	translate([0,mounting_plate_length,0]) rotor_bottom();
	translate([2*house_inner_rad,mounting_plate_length,0]) rotor_top();
	for(i=[0:roller_count-1])
		translate([i*(print_clearance+2*roller_middle_rad),2*print_clearance+2*house_inner_rad+0.5*mounting_plate_length+roller_middle_rad,0]) roller();
	translate([print_clearance+2*mounting_plate_width+clamp_width,mounting_plate_length,2*h-mounting_plate_thickness]) rotate([0,180,0]) clamp_top();
		
}

if(1 == mode) assembled();
if(2 == mode) exploded();
if(3 == mode) printable();
if(4 == mode) house();
if(5 == mode) rotor_top();
if(6 == mode) rotor_bottom();
if(7 == mode) roller();
if(8 == mode) mirror([0,0,1]) lid();
if(9 == mode) rotate([180,0,0]) clamp_top();


